/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @typescript-eslint/no-explicit-any */
import { infer } from './../infer/index.js';
/**
 * create garda procedure
 */
export function createProcedure(context, fn, options = {}) {
    // keep original context for reset
    /** actively used context */
    let _context = context;
    // recursive pipe generator
    const generatePipe = (proc, pipe) => {
        // pipe injector for context and previous value
        const injector = (...input) => {
            const pipeReturn = proc(...input);
            const value = pipe({ ...pipeReturn.context, value: pipeReturn.value }, ...input);
            return { context: pipeReturn.context, value };
        };
        const pipes = Object.assign((...input) => {
            const { context, value } = proc(...input);
            const finalValue = pipe({ ...context, value }, ...input);
            if (typeof options.onAfterCalled === 'function') {
                options.onAfterCalled(finalValue);
            }
            return finalValue;
        }, {
            pipe: (pipeFn) => {
                return generatePipe(injector, pipeFn);
            },
            infer: (inferFn) => {
                // eslint-disable-next-line @typescript-eslint/no-unused-vars
                return generatePipe(injector, ({ value }, ..._) => {
                    return inferFn(infer(value));
                });
            },
            inferAsync: (inferFn) => {
                // eslint-disable-next-line @typescript-eslint/no-unused-vars
                return generatePipe(injector, async ({ value }, ..._) => {
                    const awaitedValue = await Promise.resolve(value);
                    return inferFn(infer(awaitedValue));
                });
            },
            parse: (parser) => {
                // eslint-disable-next-line @typescript-eslint/no-unused-vars
                return generatePipe(injector, ({ value }, ..._) => {
                    return parser(value);
                });
            },
            parseAsync: (parser) => {
                // eslint-disable-next-line @typescript-eslint/no-unused-vars
                return generatePipe(injector, async ({ value }, ..._) => {
                    const awaitedValue = await Promise.resolve(value);
                    return parser(awaitedValue);
                });
            },
            inject: (ctx) => {
                // set context
                if (typeof ctx === 'function') {
                    if (Object.hasOwn(ctx, 'get') && Object.hasOwn(ctx, 'procedure')) {
                        _context = ctx;
                    }
                    else {
                        const _ctx = _context();
                        ctx(_ctx);
                        _context = () => _ctx;
                    }
                }
                else
                    _context = () => ctx;
                // passthrough previous pipe
                return generatePipe(proc, pipe);
            },
            resetContext: () => {
                _context = context;
                return generatePipe(proc, pipe);
            },
        });
        // return as new procedure
        return pipes;
    };
    // create initial injector
    const injector = (...input) => {
        const ctx = _context();
        if (typeof options.onBeforeCalled === 'function') {
            options.onBeforeCalled(input);
        }
        return {
            context: ctx,
            value: fn(ctx, ...input),
        };
    };
    // prepare caller
    const procedure = Object.assign((...input) => {
        const ctx = _context();
        if (typeof options.onBeforeCalled === 'function') {
            options.onBeforeCalled(input);
        }
        const value = fn(ctx, ...input);
        if (typeof options.onAfterCalled === 'function') {
            options.onAfterCalled(value);
        }
        return value;
    }, {
        // create first pipe function
        pipe: (pipeFn) => {
            return generatePipe(injector, pipeFn);
        },
        // create first infer function
        infer: (inferFn) => {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            return generatePipe(injector, ({ value }, ..._) => {
                return inferFn(infer(value));
            });
        },
        inferAsync: (inferFn) => {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            return generatePipe(injector, async ({ value }, ..._) => {
                const awaitedValue = await Promise.resolve(value);
                return inferFn(infer(awaitedValue));
            });
        },
        parse: (parser) => {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            return generatePipe(injector, ({ value }, ..._) => {
                return parser(value);
            });
        },
        parseAsync: (parser) => {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            return generatePipe(injector, async ({ value }, ..._) => {
                const awaitedValue = await Promise.resolve(value);
                return parser(awaitedValue);
            });
        },
        inject: (ctx) => {
            // set context
            if (typeof ctx === 'function') {
                if (Object.hasOwn(ctx, 'get') && Object.hasOwn(ctx, 'procedure')) {
                    _context = ctx;
                }
                else {
                    const _ctx = _context();
                    ctx(_ctx);
                    _context = () => _ctx;
                }
            }
            else
                _context = () => ctx;
            // create new procedure
            return createProcedure(_context, fn, options);
        },
        resetContext: () => {
            _context = context;
            return createProcedure(_context, fn, options);
        },
    });
    return procedure;
}
/** Create garda context */
export function createContext(context, options = {}) {
    const { immidiate, ...procOptions } = options;
    if (immidiate === true) {
        let $ctx = context();
        const sharedContext = Object.assign(() => $ctx, {
            get: () => $ctx,
            set(ctx) {
                $ctx = ctx;
            },
            update(fn) {
                fn($ctx);
            },
            reset() {
                $ctx = context();
            },
            procedure(fn) {
                return createProcedure(() => $ctx, fn, procOptions);
            },
        });
        return sharedContext;
    }
    else {
        let $ctx = null;
        const contextUpdate = new Set();
        const _context = () => {
            if ($ctx !== null)
                return $ctx;
            const _ctx = context();
            for (const fn of contextUpdate.values()) {
                fn(_ctx);
            }
            return _ctx;
        };
        const isolatedContext = Object.assign(() => _context(), {
            get: () => _context(),
            set(ctx) {
                $ctx = ctx;
                contextUpdate.clear();
            },
            update(fn) {
                if ($ctx !== null)
                    fn($ctx);
                else
                    contextUpdate.add(fn);
            },
            reset() {
                $ctx = null;
                contextUpdate.clear();
            },
            procedure(fn) {
                return createProcedure(() => _context(), fn, procOptions);
            },
        });
        return isolatedContext;
    }
}
