import type { PermitNodes, PermitSchema, SchemaTree } from '../types.js';
import { PermitNode, SchemaNode } from '../node.js';
export declare class Permit<T extends SchemaTree<any>> {
    /** Access Schema */
    readonly on: PermitSchema<T>;
    private _nodes;
    private schema;
    private scopes;
    constructor(schema: T, scopes?: string[]);
    static getNodeScope(scope: string | SchemaNode | PermitNode): string;
    /** get nodes that has active access or granted */
    get granted(): PermitNode[];
    /** get nodes that is inactive or denied access */
    get denied(): PermitNode[];
    /** get list of permit node */
    get nodes(): PermitNode[];
    /** reset scope to default state */
    reset(): void;
    /** grant access to scope */
    grant(...scopes: (string | SchemaNode)[]): void;
    /** deny access to scope */
    deny(...scopes: (string | SchemaNode | PermitNode)[]): void;
    ableTo(scope: string | SchemaNode | PermitNode): boolean;
    get(scope: string | SchemaNode): PermitNode | undefined;
    has(scope: string | SchemaNode | PermitNode): boolean;
    child(scope: string): PermitNode[];
    /** loop through each node */
    $each(fn: (node: PermitNode) => void): void;
}
export declare class PermitBranch<T extends Record<string, any>> {
    private $tree;
    constructor(nodes: PermitNodes<T>);
    $each(fn: (node: PermitNode) => void): void;
    get $able(): boolean;
    get $doable(): boolean;
    get $disable(): boolean;
    get $nodes(): PermitNode[];
    get $granted(): PermitNode[];
    get $denied(): PermitNode[];
}
