import { SchemaNode } from '../node.js';
import { Permit } from './permit.js';
import { loopSchema, makeScope } from '../utils.js';
export class SchemaBranch {
    $schema;
    _meta;
    constructor(schema, scope = '', spacer = '.') {
        this.$schema = schema;
        this._meta = {
            spacer: spacer,
            scope: scope,
        };
    }
    toJSON() {
        const branch = [];
        for (const key in this) {
            if (Object.prototype.hasOwnProperty.call(this, key)) {
                const node = this[key];
                if (typeof key === 'string' && !key.startsWith('$') && !key.startsWith('_')) {
                    branch.push([key, node]);
                }
            }
        }
        return Object.fromEntries(branch);
    }
    get $meta() {
        return this._meta;
    }
    get $scope() {
        return this._meta.scope;
    }
    $each(nodeHook, branchHook = () => undefined) {
        loopSchema(this.$schema, nodeHook, (node, key) => {
            branchHook(node, key);
            node.$each(nodeHook);
        });
    }
    $refactor(prefix = '', spacer = '.') {
        const branchSpacer = this._meta.spacer;
        const isSpacerSame = this._meta.spacer === spacer;
        if (!isSpacerSame) {
            this._meta.scope = this._meta.scope.replaceAll(branchSpacer, spacer);
        }
        this._meta = {
            spacer,
            scope: makeScope(spacer, [prefix, this._meta.scope]),
        };
        loopSchema(this.$schema, (node) => {
            if (!isSpacerSame)
                node.$scope = node.$scope.replaceAll(branchSpacer, spacer);
            node.$scope = makeScope(spacer, [prefix, node.$scope]);
        }, (node) => {
            node.$refactor(prefix, spacer);
        });
    }
    $export() {
        const tree = {};
        loopSchema(this.$schema, (node, key) => {
            tree[key] = { $scope: node.$scope, $description: node.$description };
        }, (node, key) => {
            const branch = node.$export();
            tree[key] = branch.schema;
        });
        return {
            schema: tree,
            meta: this._meta,
        };
    }
    $model() {
        const schema = JSON.stringify(this.$schema);
        return JSON.parse(schema);
    }
    $permit(scopes) {
        const _scopes = scopes.map((s) => Permit.getNodeScope(s));
        return new Permit(this.$schema, _scopes);
    }
}
