import { Permit } from './branch/permit.js';
import type { GeneratedSchema, Schema, SchemaOptions, SchemaScope, SchemaTree } from './types.js';
export type { Permit, PermitBranch } from './branch/permit.js';
export type { SchemaNode, PermitNode } from './node.js';
export type { Schema, SchemaOptions, SchemaTree, UnionScopes } from './types.js';
/**
 * create access control schema from tree model. Return generated access control.
 *
 * ```js
 *
 * 	const schema = createSchema({
 * 		tweet: {
 * 			create: "create a blog post",
 * 			edit: "edit a blog post",
 * 			delete: "delete a blog post"
 * 		}
 * 	}, { prefix: 'com.tweeter', spacer: ':' })
 *
 * 	console.log(schema.tweet.create.$scope)
 * 	// output: 'com.tweeter:tweet:create'
 *
 *
 * ```
 */
export declare function createSchema<T extends SchemaTree<string>>(model: T, options?: Partial<SchemaOptions>): Schema<GeneratedSchema<T>>;
/**
 * create permit object from exported schema.
 * usefull for ssr application when only need to create a permit on requested page.
 *
 * ```ts
 *
 * // on server side, send json data of schema access
 * const blogPageSchema = schema.blog.export().schema
 * const json = JSON.stringify(blogPageSchema)
 * type BlogPageSchema = typeof blogPageSchema
 *
 * // on client, parse json data
 * const schema = JSON.parse(json)
 * const permit = createPermit<BlogPageSchema>(schema, [])
 *
 *
 * ```
 */
export declare function createPermit<T extends SchemaTree<SchemaScope>>(schema: T, scopes: string[]): Permit<T>;
